import bisect

try:
    import bpy
    from .. import utils
except ModuleNotFoundError as e:
    import utils


def cal_scale_and_offset(staff_lines, strokes):
    # print('cal_scale_and_offset, {},\n {}'.format(staff_lines, strokes))
    first_staff_line = staff_lines[0]
    sec_staff_line = staff_lines[1]
    first_svg_staff_line = strokes[0]
    sec_svg_staff_line = strokes[1]
    x_scale = (first_svg_staff_line[-1][0] - first_svg_staff_line[0][0]) / (
            first_staff_line[1][0] - first_staff_line[0][0])
    y_scale = (sec_svg_staff_line[0][-1] - first_svg_staff_line[0][-1]) / (
            sec_staff_line[0][1] - first_staff_line[0][1])
    offset_x = first_svg_staff_line[0][0] - first_staff_line[0][0] * x_scale
    offset_y = sec_svg_staff_line[0][-1] - sec_staff_line[0][1] * y_scale
    return (x_scale, y_scale), (offset_x, offset_y)


def calculate_curve_section_length(curve):
    tmp_curve = utils.copy_obj(curve)
    tmp_curve.data.splines[0].resolution_u = 64
    for obj in bpy.data.objects:
        obj.select_set(False)
    bpy.context.view_layer.objects.active = tmp_curve
    tmp_curve.select_set(True)
    bpy.ops.object.convert(target='MESH')
    section_lens = []
    if tmp_curve.type == 'MESH':
        mesh = tmp_curve.data
        acc = 0
        last_co = mesh.vertices[0].co
        for vertex in mesh.vertices:
            acc += (vertex.co - last_co).length
            last_co = vertex.co
            if vertex.co.z == 0:
                section_lens.append(acc)
    bpy.data.objects.remove(tmp_curve, do_unlink=True)
    return section_lens


def calc_frame(x, x_to_frame):
    keys = list(x_to_frame.keys())
    keys.sort()
    index = bisect.bisect_left(keys, x)
    if index <= 0:
        return 0
    if index >= len(keys):
        return x_to_frame[keys[-1]]
    left_x = keys[index - 1]
    right_x = keys[index]
    left_frame = x_to_frame[left_x]
    right_frame = x_to_frame[right_x]
    division = (right_frame - left_frame) / (right_x - left_x)
    return division * (x - left_x) + left_frame
