import json

try:
    import bpy
    import mathutils
    from . import scene
    from .. import utils
    from .. import config
    from ..tools import bouncecurve as bc
    from ..bouncecurve import bcutils
except ModuleNotFoundError as e:
    import scene
    import utils
    import config
    import bcutils
    from tools import bouncecurve as bc


class GenerateCameraOperator(bpy.types.Operator):
    bl_idname = "object.bc_gen_camera_animation"
    bl_label = "生成摄像机动画"
    bl_options = {'REGISTER', 'UNDO'}

    def execute(self, context):
        camera = context.scene.bc_camera
        if camera is None:
            self.report({'ERROR'}, '请先选中摄像机')
            return {'FINISHED'}
        data = context.scene.bc_score_data
        try:
            j = json.loads(data)
        except:
            self.report({'ERROR'}, '乐谱数据丢失，请重新生成乐谱')
            return {'FINISHED'}
        scale = j.get('scale')
        offset = j.get('offset')
        parts = j.get('parts', [])
        obj_score = context.scene.bc_obj_svg_score
        obj_scales = obj_score.scale
        if obj_scales[0] != obj_scales[2]:
            self.report({'INFO'}, '请保证乐谱X/Y轴以等比例缩放')
            return {'FINISHED'}
        part = parts[0]
        times = [t.get('onset_time') for t in part.get('times', [])]
        note_paths = part.get('note_paths', [])
        notes = []
        for ind_note in range(len(note_paths[0])):
            for path in note_paths:
                if path[ind_note] is None:
                    continue
                notes.append(path[ind_note])
                break
        if len(times) < 1:
            self.report({'INFO'}, '乐谱数据异常1')
            return
        if len(times) != len(notes):
            print('times len: {}, chords len:{}'.format(len(times), len(notes)))
            self.report({'INFO'}, '乐谱数据异常2')
            return
        tmp_points = []
        for note_ind, note in enumerate(notes):
            tmp_points.append((mathutils.Vector(note) * mathutils.Vector(scale) + mathutils.Vector(
                offset)) * mathutils.Vector((
                obj_scales[0], obj_scales[2])))
        camera.animation_data_clear()
        location = camera.location.copy()
        utils.set_keyframe(camera, 'location', 0, location)
        for ind in range(0, len(times), 10):
            loc = (tmp_points[ind][0] + location[0] - tmp_points[0][0], location[1], location[2])
            utils.set_keyframe(camera, 'location', int(times[ind] * bpy.context.scene.render.fps), loc)
        for curve in camera.animation_data.action.fcurves:
            if curve.data_path == 'location':
                for kf in curve.keyframe_points:
                    kf.interpolation = 'LINEAR'
        return {'FINISHED'}
